package main

import (
	"bytes"
	"crypto/ed25519"
	"crypto/rand"
	"encoding/ascii85"
	"encoding/base64"
	"encoding/hex"
	"encoding/json"
	"fmt"
	"github.com/urfave/cli/v2"
	"os"
	"path"
)

func KeyGenerate(c *cli.Context) error {
	cp := GetKeyStorePath(c)
	name := c.String("name")
	if name == "" {
		return fmt.Errorf("flag 'name' should not be empty")
	}
	fpath := path.Join(cp, fmt.Sprintf("%s.key.json", name))
	_, err := os.Stat(fpath)
	if err == nil {
		return fmt.Errorf("key %s already exists", name)
	} else {
		if !os.IsNotExist(err) {
			return fmt.Errorf("failed use stat to check key file '%s.key.json' existance: %w", name, err)
		}
	}
	pubkey, prikey, err := ed25519.GenerateKey(rand.Reader)
	if err != nil {
		return fmt.Errorf("failed to generate ed25519 key: %w", err)
	}
	EncodedPubKey := base64.URLEncoding.EncodeToString(pubkey)
	EncodedPriKey := base64.URLEncoding.EncodeToString(prikey)
	jdat := KeyStore{
		PublicKey:  EncodedPubKey,
		PrivateKey: EncodedPriKey,
	}
	jstr, err := json.Marshal(jdat)
	if err != nil {
		return fmt.Errorf("failed to marshal key store: %w", err)
	}
	err = os.WriteFile(fpath, jstr, 0660)
	if err != nil {
		return fmt.Errorf("failed to write key store: %w", err)
	}
	fmt.Println("Done.")
	return nil
}

func PrintPublicKey(c *cli.Context) error {
	pub, _, err := LoadKey(c)
	if err != nil {
		return err
	}
	fmt.Println("==== Base64 URL Encoding ====")
	fmt.Println(base64.URLEncoding.EncodeToString(pub))
	fmt.Println("==== Base64 Std Encoding ====")
	fmt.Println(base64.StdEncoding.EncodeToString(pub))
	fmt.Println("==== Hex Encoding ====")
	fmt.Println(hex.EncodeToString(pub))
	fmt.Println("==== Ascii85 Encoding ====")
	buf := bytes.NewBuffer([]byte{})
	a85enc := ascii85.NewEncoder(buf)
	_, err = a85enc.Write(pub)
	if err != nil {
		return fmt.Errorf("failed to write to ascii85 encoder: %w", err)
	}
	err = a85enc.Close()
	if err != nil {
		return fmt.Errorf("failed to close ascii85 encoder: %w", err)
	}
	fmt.Println(buf.String())
	return nil
}
